@@ -20,14 +20,15 @@
 #'
 #' \code{\link{brglmFit}} is a fitting function for \code{\link{glm}}
 #' that fits generalized linear models using implicit and explicit
-#' bias reduction methods. Currently supported methods include the
-#' mean bias-reducing adjusted scores approach in Firth (1993) and
-#' Kosmidis \& Firth (2009), the median bias-reduction adjusted scores
-#' approach in Kenne et al. (2016), the correction of the asymptotic
-#' bias in Cordeiro & McCullagh (1991), and maximum likelihood.
-#' Estimation is performed using a quasi Fisher scoring iteration,
-#' which in the case of mean-bias reduction resembles an iterative
-#' correction of the asymptotic bias of the Fisher scoring iterates.
+#' bias reduction methods (Kosmidis, 2014). Currently supported
+#' methods include the mean bias-reducing adjusted scores approach in
+#' Firth (1993) and Kosmidis \& Firth (2009), the median
+#' bias-reduction adjusted scores approach in Kenne et al. (2016), the
+#' correction of the asymptotic bias in Cordeiro & McCullagh (1991),
+#' and maximum likelihood. Estimation is performed using a quasi
+#' Fisher scoring iteration, which, in the case of mean-bias
+#' reduction, resembles an iterative correction of the asymptotic bias
+#' of the Fisher scoring iterates.
 #'
 #' @inheritParams stats::glm.fit
 #' @aliases brglm_fit
@@ -50,23 +51,38 @@
 #'     \code{poisson}. Either \code{NULL} (no effect) or a vector that
 #'     indicates which counts must be treated as a group. See Details
 #'     for more information and \code{\link{brmultinom}}.
+#' @param singular.ok logical. If \code{FALSE}, a singular model is an
+#'     error.
 #' @param ... arguments to be used to form the default 'control'
 #'     argument if it is not supplied directly.
 #'
 #' @details
 #'
-#' Implicit and explicit bias reduction methods are described in
-#' detail in Kosmidis (2014). The quasi (or modified) Fisher scoring
-#' iteration is described in Kosmidis (2010) and is based on the
-#' iterative correction of the asymptotic bias of the Fisher scoring
-#' iterates. A mathematical description of the supported adjustments
-#' and the quasi Fisher scoring iteration is provided in the iteration
-#' vignette (see,
+#' A detailed description of the supported adjustments and the quasi
+#' Fisher scoring iteration is given in the iteration vignette (see,
 #' \url{https://cran.r-project.org/package=brglm2/vignettes/iteration.pdf}).
-#' A quick description of the quasi Fisher scoring iteration is also
+#' A shorter description of the quasi Fisher scoring iteration is also
 #' given in one of the vignettes of the *enrichwith* R package (see,
 #' \url{https://cran.r-project.org/package=enrichwith/vignettes/bias.html}).
+#' Kosmidis and Firth (2010) describe a parallel quasi Newton-Raphson
+#' iteration with the same stationary point.
 #'
+#' In the special case of generalized linear models for binomial,
+#' Poisson and multinomial responses, the adjusted score equations
+#' approach returns estimates with improved frequentist properties,
+#' that are also always finite, even in cases where the maximum
+#' likelihood estimates are infinite (e.g. complete and quasi-complete
+#' separation in multinomial regression; see also
+#' \code{\link{detect_separation}} and
+#' \code{\link{check_infinite_estimates}} for pre-fit and post-fit
+#' methods for the detection of infinite estimates in binomial
+#' response generalized linear models).
+#'
+#' The type of the bias-reducing adjustment to be used is specified
+#' through the \code{type} argument (see \code{\link{brglmControl}}
+#' for details). The default is to use the mean bias-reducing
+#' adjustsments in Firth (1993) and Kosmidis \& Firth (2009)
+#' (\code{type = "AS_mean"}).
 #'
 #' The null deviance is evaluated based on the fitted values using the
 #'     method specified by the \code{type} argument (see
@@ -193,7 +209,7 @@ brglmFit <- function (x, y, weights = rep(1, nobs), start = NULL, etastart = NUL
                       mustart = NULL, offset = rep(0, nobs), family = gaussian(),
                       control = list(), intercept = TRUE,
                       ## Arguments that glm will not use in its call to brglmFit (be wise with defaults!)
-                      fixed_totals = NULL)
+                      fixed_totals = NULL, singular.ok = TRUE)
 {
 
     trace_iteration <- function() {
@@ -460,6 +476,24 @@ brglmFit <- function (x, y, weights = rep(1, nobs), start = NULL, etastart = NUL
         })
     }
 
+    AS_mixed_adjustment <- function(pars, level = 0, fit = NULL) {
+        if (is.null(fit)) {
+            fit <- key_quantities(pars, y = y, level = level, qr = TRUE)
+        }
+        with(fit, {
+            if (level == 0) {
+                hatvalues <- hat_values(pars, fit = fit)
+                ## Use only observations with keep = TRUE to ensure that no division with zero takes place
+                return(.colSums(0.5 * hatvalues * d2mus/d1mus * x, nobs, nvars, TRUE))
+            }
+            if (level == 1) {
+                s1 <- sum(weights^3 * d3afuns, na.rm = TRUE)
+                s2 <- sum(weights^2 * d2afuns, na.rm = TRUE)
+                return(nvars/(2 * dispersion) + s1/(6 * dispersion^2 * s2))
+            }
+        })
+    }
+
     customTransformation <- is.list(control$transformation) & length(control$transformation) == 2
     if (customTransformation) {
         transformation0 <- control$transformation
@@ -472,6 +506,7 @@ brglmFit <- function (x, y, weights = rep(1, nobs), start = NULL, etastart = NUL
                             "correction" = AS_mean_adjustment,
                             "AS_mean" = AS_mean_adjustment,
                             "AS_median" = AS_median_adjustment,
+                            "AS_mixed" = AS_mixed_adjustment,
                             "ML" = function(pars, ...) 0)
 
     ## compute_step_components does everything on the scale of the /transformed/ dispersion
@@ -617,7 +652,11 @@ brglmFit <- function (x, y, weights = rep(1, nobs), start = NULL, etastart = NUL
         ## Detect aliasing
         qrx <- qr(x)
         rank <- qrx$rank
-        is_full_rank <- all.equal(rank, nvars, tolerance = 1e-06)
+        is_full_rank <- rank == nvars
+
+        if (!singular.ok && !is_full_rank) {
+            stop("singular fit encountered")
+        }
         if (!isTRUE(is_full_rank)) {
             aliased <- qrx$pivot[seq.int(qrx$rank + 1, nvars)]
             X_all <- x
